#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# (C) Copyright 2008 Kulbir Saini <kulbirsaini@students.iiit.ac.in>
#

from config import readMainConfig, readStartupConfig
import logging
import os
import stat
import sys
import urlparse

# To modify configuration parameters, see /etc/intelligentmirror.conf .
# Read config file using Yum's config parsers.
mainconf = readMainConfig(readStartupConfig('/etc/intelligentmirror.conf', '/'))

# Global hard coded variables
rpm_files = ['.rpm']
deb_files = ['.deb']
format = '%-7s %3s %s'

# Global variables from config file.
cache_host =  mainconf.cache_host
logfile = mainconf.logfile

# RPM related variables.
enable_rpm_cache = int(mainconf.enable_rpm_cache)

# Deb related variables.
enable_deb_cache = int(mainconf.enable_deb_cache)

def set_logging():
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)s %(message)s',
                        filename=logfile,
                        filemode='a')
    return logging.info

def squid_part():
    """This function will tap requests from squid. If the request is for rpm/deb
    packages, they will be modified to look as http://cache_host/[rpm|deb]/package.
    Finally this function will flush a new url if package was an rpm/deb or a
    blank line otherwise."""
    while True:
        # Read url from stdin ( this is provided by squid)
        url = sys.stdin.readline().strip().split(' ')
        new_url = '\n';
        # Retrieve the basename from the request url
        path = urlparse.urlsplit(url[0])[2]
        query = os.path.basename(path)
        if enable_rpm_cache:
            for file in rpm_files:
                if query.endswith(file):
                    # This signifies that URL is a rpm package
                    #log(format%('URL_HIT', 'RPM', query + ' ' + url[0]))
                    new_url = 'http://' + cache_host + '/rpm/' + query + new_url
                    log(format%('NEW_URL', 'RPM', query + ' ' + new_url.strip('\n')))
                    break
        if enable_deb_cache:
            for file in deb_files:
                if query.endswith(file):
                    # This signifies that URL is a deb package
                    #log(format%('URL_HIT', 'DEB', query + ' ' + url[0]))
                    new_url = 'http://' + cache_host + '/deb/' + query + new_url
                    log(format%('NEW_URL', 'DEB', query + ' ' + new_url.strip('\n')))
                    break
        # Flush the new url to stdout for squid to process
        sys.stdout.write(new_url)
        sys.stdout.flush()

if __name__ == '__main__':
    # Invoke logger
    log = set_logging()
    # Invoke squid store url rewriter
    squid_part()
